@@ -10,13 +10,15 @@ Released under GPL version 3 or later.
 module Hledger.UI.Main where
 
 -- import Control.Applicative
--- import Control.Lens ((^.))
+-- import Lens.Micro.Platform ((^.))
 import Control.Monad
 -- import Control.Monad.IO.Class (liftIO)
 -- import Data.Default
 -- import Data.Monoid              -- 
 import Data.List
 import Data.Maybe
+-- import Data.Text (Text)
+import qualified Data.Text as T
 -- import Data.Time.Calendar
 import Safe
 import System.Exit
@@ -30,8 +32,8 @@ import Hledger.UI.UIOptions
 import Hledger.UI.UITypes
 -- import Hledger.UI.UIUtils
 import Hledger.UI.Theme
-import Hledger.UI.AccountsScreen as AS
-import Hledger.UI.RegisterScreen as RS
+import Hledger.UI.AccountsScreen
+import Hledger.UI.RegisterScreen
 
 ----------------------------------------------------------------------
 
@@ -42,8 +44,11 @@ main = do
   run opts
     where
       run opts
-        | "help" `inRawOpts` (rawopts_ $ cliopts_ opts)            = putStr (showModeHelp uimode) >> exitSuccess
-        | "version" `inRawOpts` (rawopts_ $ cliopts_ opts)         = putStrLn prognameandversion >> exitSuccess
+        | "h"               `inRawOpts` (rawopts_ $ cliopts_ opts) = putStr (showModeUsage uimode) >> exitSuccess
+        | "help"            `inRawOpts` (rawopts_ $ cliopts_ opts) = printHelpForTopic (topicForMode uimode) >> exitSuccess
+        | "man"             `inRawOpts` (rawopts_ $ cliopts_ opts) = runManForTopic (topicForMode uimode) >> exitSuccess
+        | "info"            `inRawOpts` (rawopts_ $ cliopts_ opts) = runInfoForTopic (topicForMode uimode) >> exitSuccess
+        | "version"         `inRawOpts` (rawopts_ $ cliopts_ opts) = putStrLn prognameandversion >> exitSuccess
         | "binary-filename" `inRawOpts` (rawopts_ $ cliopts_ opts) = putStrLn (binaryfilename progname)
         | otherwise                                                = withJournalDoUICommand opts runBrickUi
 
@@ -71,11 +76,18 @@ runBrickUi uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}} j = do
     uopts' = uopts{
       cliopts_=copts{
          reportopts_= ropts{
-            -- ensure depth_ also reflects depth: args
-            depth_=depthfromoptsandargs,
-            -- remove depth: args from query_
-            query_=unwords $ -- as in ReportOptions, with same limitations
-                   [v | (k,v) <- rawopts_ copts, k=="args", not $ "depth" `isPrefixOf` v]
+            -- incorporate any depth: query args into depth_,
+            -- any date: query args into period_
+            depth_ =depthfromoptsandargs,
+            period_=periodfromoptsandargs,
+            query_ =unwords -- as in ReportOptions, with same limitations
+                    [v | (k,v) <- rawopts_ copts, k=="args", not $ any (`isPrefixOf` v) ["depth","date"]],
+            -- always disable boring account name eliding, unlike the CLI, for a more regular tree
+            no_elide_=True,
+            -- show items with zero amount by default, unlike the CLI
+            empty_=True,
+            -- show historical balances by default, unlike the CLI
+            balancetype_=HistoricalBalance
             }
          }
       }
@@ -83,6 +95,9 @@ runBrickUi uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}} j = do
         q = queryFromOpts d ropts
         depthfromoptsandargs = case queryDepth q of 99999 -> Nothing
                                                     d     -> Just d
+        datespanfromargs = queryDateSpan (date2_ ropts) $ fst $ parseQuery d (T.pack $ query_ ropts)
+        periodfromoptsandargs =
+          dateSpanAsPeriod $ spansIntersect [periodAsDateSpan $ period_ ropts, datespanfromargs]
 
     -- XXX move this stuff into Options, UIOpts
     theme = maybe defaultTheme (fromMaybe defaultTheme . getTheme) $
@@ -90,48 +105,45 @@ runBrickUi uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}} j = do
     mregister = maybestringopt "register" $ rawopts_ copts
 
     (scr, prevscrs) = case mregister of
-      Nothing   -> (AS.screen, [])
+      Nothing   -> (accountsScreen, [])
       -- with --register, start on the register screen, and also put
       -- the accounts screen on the prev screens stack so you can exit
       -- to that as usual.
-      Just apat -> (rsSetCurrentAccount acct RS.screen, [ascr'])
+      Just apat -> (rsSetAccount acct False registerScreen, [ascr'])
         where
           acct = headDef
                  (error' $ "--register "++apat++" did not match any account")
-                 $ filter (regexMatches apat) $ journalAccountNames j
+                 $ filter (regexMatches apat . T.unpack) $ journalAccountNames j
           -- Initialising the accounts screen is awkward, requiring
-          -- another temporary AppState value..
+          -- another temporary UIState value..
           ascr' = aScreen $
-                  AS.initAccountsScreen d $
-                  AppState{
+                  asInit d True $
+                  UIState{
                     aopts=uopts'
                    ,ajournal=j
-                   ,aScreen=asSetSelectedAccount acct AS.screen
+                   ,aScreen=asSetSelectedAccount acct accountsScreen
                    ,aPrevScreens=[]
+                   ,aMode=Normal
                    }
   
-    st = (sInitFn scr) d
-         AppState{
+    ui = (sInit scr) d True
+         UIState{
             aopts=uopts'
            ,ajournal=j
            ,aScreen=scr
            ,aPrevScreens=prevscrs
+           ,aMode=Normal
            }
-         
-    app :: App (AppState) V.Event
-    app = App {
+
+    brickapp :: App (UIState) V.Event Name
+    brickapp = App {
         appLiftVtyEvent = id
       , appStartEvent   = return
       , appAttrMap      = const theme
       , appChooseCursor = showFirstCursor
-      , appHandleEvent  = \st ev -> sHandleFn (aScreen st) st ev
-      , appDraw         = \st -> sDrawFn (aScreen st) st
-         -- XXX bizarro. removing the st arg and parameter above,
-         -- which according to GHCI does not change the type,
-         -- causes "Exception: draw function called with wrong screen type"
-         -- on entering a register. Likewise, removing the st ev args and parameters
-         -- causes an exception on exiting a register.
+      , appHandleEvent  = \ui ev -> sHandle (aScreen ui) ui ev
+      , appDraw         = \ui    -> sDraw   (aScreen ui) ui
       }
 
-  void $ defaultMain app st
+  void $ defaultMain brickapp ui
 
